
# This file was generated by 'versioneer.py' (0.18) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

import json

version_json = '''
{
 "branch": "0.11.0)",
 "date": "2020-05-10T18:17:21+0100",
 "dirty": false,
 "error": null,
 "full-revisionid": "3028710b5652aa98dd3409ee669b7e6e428d2525",
 "repository": "https://github.com/metawards/MetaWards",
 "version": "0.11.0"
}
'''  # END VERSION_JSON


def get_versions():
    return json.loads(version_json)
